<?php

namespace Schaffen\Gef\Http\Controllers;

use App\Http\Controllers\Controller;
use App\Http\Utils\Response;
use App\Models\Project\Obra;
use App\Models\SCHModel;
use Illuminate\Http\Request;

use Schaffen\Gef\Http\Requests;
use Prettus\Service\Contracts\ServiceInterface;
use Prettus\Service\Exceptions\ServiceException;
use Schaffen\Gef\Http\Requests\ApontamentoVeiculoUsoCreateRequest;
use Schaffen\Gef\Http\Requests\ApontamentoVeiculoUsoUpdateRequest;
use Schaffen\Gef\Models\ApontamentoVeiculoUso;
use Schaffen\Gef\Repositories\ApontamentoVeiculoUsoRepository;
use Schaffen\Gef\Services\ApontamentoVeiculoUsoService;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;

/**
 * Class ApontamentoVeiculoUsosController.
 *
 * @package namespace Schaffen\Gef\Http\Controllers;
 */
class ApontamentoVeiculoUsosController extends Controller
{
   /**
    * @var ApontamentoVeiculoUsoRepository
    */
   protected $repository;

   /**
    * @var ApontamentoVeiculoUsoService
    */
   protected $service;

   /**
    * ApontamentoVeiculoUsosController constructor.
    *
    * @param ApontamentoVeiculoUsoRepository $repository
    * @param ApontamentoVeiculoUsoService $service
    */
   public function __construct(ApontamentoVeiculoUsoRepository $repository, ApontamentoVeiculoUsoService $service)
   {
      $this->repository = $repository;
      $this->service = $service;
   }

   /**
    * Display a listing of the resource.
    *
    * @return \Illuminate\Http\Response
    */
   public function index(Request $request)
   {
      $this->repository->pushCriteria(new \Schaffen\Gef\Criterias\RequestPrefixCriteria($request, ApontamentoVeiculoUso::$prefix));
      $apontamentoVeiculoUsos = $this->repository->all();

      $ret['cols'] = array();
      array_push($ret['cols'], ['field' => "codigo", 'title' => "Código", 'dataType' => 'id', 'filter' => ['codigo' => "text"], 'sortable' => "codigo", 'show' => true]);
      array_push($ret['cols'], ['field' => "veiculo", 'title' => "Veiculo", 'dataType' => 'text', 'filter' => ['veiculo' => "text"], 'sortable' => "veiculo", 'show' => true]);
      array_push($ret['cols'], ['field' => "km_inicial", 'title' => "km inicial", 'dataType' => 'numeric', 'filter' => ['km_inicial' => "number"], 'sortable' => "km_inicial", 'show' => true]);
      array_push($ret['cols'], ['field' => "km_final", 'title' => "km final", 'dataType' => 'numeric', 'filter' => ['km_final' => "number"], 'sortable' => "km_final", 'show' => true]);
      array_push($ret['cols'], ['field' => "data_hora_uso_inical", 'title' => "Data inical", 'dataType' => 'data', 'filter' => ['data_hora_uso_inical' => "text"], 'sortable' => "data_hora_uso_inical", 'show' => true]);
      array_push($ret['cols'], ['field' => "data_hora_uso_final", 'title' => "Data final", 'dataType' => 'data', 'filter' => ['data_hora_uso_final' => "text"], 'sortable' => "data_hora_uso_final", 'show' => true]);
      array_push($ret['cols'], ['field' => "motorista", 'title' => "Motorista", 'dataType' => 'text', 'filter' => ['motorista' => "text"], 'sortable' => "motorista", 'show' => true]);
      array_push($ret['cols'], ['field' => "action", 'title' => "Opções", 'dataType' => 'command', 'show' => true]);
      $ret['rows'] = $apontamentoVeiculoUsos["data"];
      return response()->json([
         'data' => $ret,
      ]);

   }

   /**
    * Store a newly created resource in storage.
    *
    * @param  ApontamentoVeiculoUsoCreateRequest $request
    *
    * @return \Illuminate\Http\Response
    *
    * @throws \Prettus\Service\Exceptions\ServiceException
    */
   public function store(ApontamentoVeiculoUsoCreateRequest $request)
   {
      $response = $this->service->updateOrCreate($request->all());
      return Response::response($response['status'], $response['message'], $response['data'], $response['errors'], $response['http_code']);
   }

   /**
    * Display the specified resource.
    *
    * @param  int $id
    *
    * @return \Illuminate\Http\Response
    */
   public function show($id)
   {
      $this->repository->setPresenter("Schaffen\\Gef\\Presenters\\ApontamentoVeiculoUsoShowPresenter");
      $apontamento_veiculo_uso = $this->repository->find($id);


      return Response::response(true, null, $apontamento_veiculo_uso, null);
   }

   /**
    * Show the form for editing the specified resource.
    *
    * @param  int $id
    *
    * @return \Illuminate\Http\Response
    */
   public function edit($id)
   {
      $apontamentoVeiculoUso = $this->repository->find($id);

      return view('apontamentoVeiculoUsos.edit', compact('apontamentoVeiculoUso'));
   }

   /**
    * Update the specified resource in storage.
    *
    * @param  ApontamentoVeiculoUsoUpdateRequest $request
    * @param  string $id
    *
    * @return Response
    *
    * @throws \Prettus\Service\Exceptions\ServiceException
    */
   public function update(ApontamentoVeiculoUsoUpdateRequest $request, $id)
   {
      $response = $this->service->updateOrCreate($request->all(), $id);
      return Response::response($response['status'], $response['message'], $response['data'], $response['errors'], $response['http_code']);
   }


   /**
    * Remove the specified resource from storage.
    *
    * @param  int $id
    *
    * @return \Illuminate\Http\Response
    */
   public function destroy($id)
   {
      $deleted = $this->repository->delete($id);

      if (request()->wantsJson()) {

         return response()->json([
            'message' => 'Apontamento de uso removido com sucesso.',
            'deleted' => $deleted,
         ]);
      }
      return Response::success('Apontamento de uso removido com sucesso.','');
   }

   public function getObraNome($nome)
   {
      $filtro = [
         SCHModel::buildFilterObject("obr_descricao", "ilike", "%{$nome}%"),
         SCHModel::buildFilterObject("obr_fk_obs_codigo", "!=", "3")
      ];

      $orderBy = [
         'col' => 'obr_descricao',
         'dir' => 'asc'
      ];

      return Response::success('', Obra::getObra($filtro, $orderBy));
   }

   public function getObra($filtro)
   {
      $filtro = json_decode($filtro);
      array_push($filtro, SCHModel::buildFilterObject("obr_fk_obs_codigo", "!=", "3"));
      return Response::success('', Obra::getObra());
   }
}
